<?php
namespace Magenest\Salesforce\Setup\Patch\Data;

use Magenest\Salesforce\Model\Connector;
use Magento\Catalog\Model\Product;
use Magento\Config\Model\ResourceModel\Config as ResourceConfig;
use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Setup\SalesSetupFactory;

class InstallAttributes implements DataPatchInterface
{
    /**
     * @var ModuleDataSetupInterface $moduleDataSetup
     */
    private $moduleDataSetup;

    /**
     * @var EavSetupFactory
     */
    private $eavSetupFactory;

    /**
     * @var CustomerSetupFactory
     */
    private $customerSetupFactory;

    /**
     * @var AttributeSetFactory
     */
    private $attributeSetFactory;

    /**
     * @var SalesSetupFactory
     */
    private $salesSetupFactory;

    /**
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param EavSetupFactory $eavSetupFactory
     * @param CustomerSetupFactory $customerSetupFactory
     * @param AttributeSetFactory $attributeSetFactory
     * @param SalesSetupFactory $salesSetupFactory
     */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory          $eavSetupFactory,
        CustomerSetupFactory     $customerSetupFactory,
        AttributeSetFactory      $attributeSetFactory,
        SalesSetupFactory        $salesSetupFactory
    ) {
        $this->moduleDataSetup      = $moduleDataSetup;
        $this->eavSetupFactory      = $eavSetupFactory;
        $this->customerSetupFactory = $customerSetupFactory;
        $this->attributeSetFactory  = $attributeSetFactory;
        $this->salesSetupFactory    = $salesSetupFactory;
    }

    /**
     * Do Upgrade
     *
     * @return void
     * @throws \Exception
     */
    public function apply()
    {
        $setup = $this->moduleDataSetup;
        $this->createCustomerAttribute($setup);
        $this->createProductAttribute($setup);
        $this->createOrderAttribute($setup);
    }

    /**
     * @param $setup
     * @return void
     * @throws \Exception
     * @throws \Zend_Validate_Exception
     */
    private function createCustomerAttribute($setup)
    {
        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

        $customerEntity = $customerSetup->getEavConfig()->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        $attributeSet     = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);

        $customerSetup->addAttribute(
            Customer::ENTITY,
            'salesforce_account_id',
            [
                'type'         => 'text',
                'label'        => 'Salesforce Account Id',
                'input'        => 'text',
                'required'     => false,
                'visible'      => false,
                'user_defined' => false,
                'position'     => 999,
                'system'       => 0,
            ]
        );

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'salesforce_account_id')
            ->addData(
                [
                    'attribute_set_id'   => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms'      => ['adminhtml_customer']
                ]
            );
        $attribute->save();
    }

    /**
     * @param $setup
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Zend_Validate_Exception
     */
    private function createProductAttribute($setup)
    {
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);
        $eavSetup->addAttribute(
            Product::ENTITY,
            'salesforce_product_id',
            [
                'type'         => 'text',
                'visible'      => false,
                'required'     => false,
                'user_defined' => false,
                'label'        => 'Salesforce Product Id',
                'system'       => false
            ]
        );

        $eavSetup->addAttribute(
            Product::ENTITY,
            'salesforce_pricebookentry_id',
            [
                'type'         => 'text',
                'visible'      => false,
                'required'     => false,
                'user_defined' => false,
                'label'        => 'Salesforce PricebookEntry Id',
                'system'       => false,
            ]
        );
    }

    /**
     * @param $setup
     * @return void
     */
    private function createOrderAttribute($setup)
    {
        /** @var \Magento\Sales\Setup\SalesSetup $salesSetup */
        $salesSetup = $this->salesSetupFactory->create(['setup' => $setup]);

        $salesSetup->addAttribute(
            Order::ENTITY,
            'salesforce_order_id',
            [
                'type'         => 'text',
                'visible'      => false,
                'required'     => false,
                'user_defined' => false,
                'label'        => 'Salesforce Order Id',
                'system'       => false,
            ]
        );

        $salesSetup->addAttribute(
            Order::ENTITY,
            'salesforce_opportunity_id',
            [
                'type'         => 'text',
                'visible'      => false,
                'required'     => false,
                'user_defined' => false,
                'label'        => 'Salesforce Opportunity Id',
                'system'       => false,
            ]
        );
    }

    /**
     * @inheritdoc
     */
    public function getAliases()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies()
    {
        return [];
    }
}
